@@ -22,7 +22,7 @@ BEGIN {
    $main_proc_id = $$; $prog_name = $0; $prog_name =~ s{^.*[\\/]}{}g;
 }
 
-our $VERSION = '1.303';
+our $VERSION = '1.304';
 $VERSION = eval $VERSION;
 
 our $tmp_dir = undef;
@@ -124,17 +124,21 @@ our $mce_spawned_ref = undef;
 
 END {
    my $_exit_status = $?;
-   MCE::Signal->_shutdown_mce($_exit_status);
-   MCE::Signal->stop_and_exit($_exit_status) if ($$ == $main_proc_id);
+
+   MCE::Signal->_shutdown_mce($_exit_status)
+      if (defined $mce_spawned_ref);
+
+   MCE::Signal->stop_and_exit($_exit_status)
+      if ($$ == $main_proc_id);
 }
 
 ###############################################################################
 ## ----------------------------------------------------------------------------
 ## Run command via the system(...) function.
 ##
-## The system function in Perl ignores SIGINT and SIGQUIT.  These 2 signals
+## The system function in Perl ignores SIGINT and SIGQUIT. These 2 signals
 ## are sent to the command being executed via system() but not back to
-## the underlying Perl script.  The code below will ensure the Perl script
+## the underlying Perl script. The code below will ensure the Perl script
 ## receives the same signal in order to raise an exception immediately
 ## after the system call.
 ##
@@ -183,6 +187,7 @@ sub sys_cmd {
       my $_is_sig      = 0;
 
       if (exists $_sig_name_lkup{$_sig_name}) {
+         $mce_spawned_ref = undef;
          $SIG{$_sig_name} = sub { };
          $_exit_status = $_is_sig = 1;
       }
@@ -190,9 +195,8 @@ sub sys_cmd {
          $_exit_status = $_sig_name if ($_sig_name ne '0');
       }
 
-      $SIG{TERM}     = sub { } if ($_sig_name ne 'TERM');
-      $SIG{__DIE__}  = sub { };
-      $SIG{__WARN__} = sub { };
+      $SIG{TERM} = sub { } if ($_sig_name ne 'TERM');
+      $SIG{__DIE__} = $SIG{__WARN__} = sub { };
 
       ## ----------------------------------------------------------------------
 
@@ -219,6 +223,9 @@ sub sys_cmd {
                elsif ($_sig_name eq 'TERM' && -f "$tmp_dir/died") {
                   $_err_msg = "caught signal '__DIE__', exiting";
                }
+               elsif ($_sig_name eq '__DIE__') {
+                  $_err_msg = "caught signal '__DIE__', exiting";
+               }
                elsif ($_sig_name ne 'PIPE') {
                   $_err_msg = "caught signal '$_sig_name', exiting";
                }
@@ -232,7 +239,7 @@ sub sys_cmd {
 
                ## Pause a bit.
                if ($_sig_name ne 'PIPE') {
-                  select(undef, undef, undef, 0.066) for (1..3);
+                  select(undef, undef, undef, 0.066) for (1..4);
                }
             }
 
@@ -292,7 +299,7 @@ sub sys_cmd {
 
       ## Exit thread/process with status.
       if ($_is_sig == 1) {
-         select(undef, undef, undef, 0.133) for (1..3);
+         select(undef, undef, undef, 0.066) for (1..8);
       }
 
       threads->exit($_exit_status) if ($has_threads && threads->can('exit'));
@@ -339,7 +346,8 @@ sub _die_handler {
 
    shift @_ if (defined $_[0] && $_[0] eq 'MCE::Signal');
 
-   local $SIG{__DIE__} = sub { };
+   CORE::die(@_) if $^S;      ## Direct to CORE::die if executing an eval
+
    local $\ = undef;
 
    ## Set $MCE::Signal::display_die_with_localtime = 1;
@@ -354,6 +362,8 @@ sub _die_handler {
    }
 
    MCE::Signal->stop_and_exit('__DIE__');
+
+   CORE::exit;
 }
 
 sub _warn_handler {
@@ -398,7 +408,7 @@ MCE::Signal - Provides tmp_dir creation & signal handling for Many-Core Engine.
 
 =head1 VERSION
 
-This document describes MCE::Signal version 1.303
+This document describes MCE::Signal version 1.304
 
 =head1 SYNOPSIS
 
